//=============================================================================
// UTPlayer.
//=============================================================================
class UTPlayer expands PlayerPawn;

var byte PlayingTrack;
var bool bIsWalkingNow,bIsCrouchingNow,bIsTypingNow,bPlayerFrozen,bUpdatingMove;
var() class<PawnCRender> RenderingClass;
var() class<UTSavedMove> SavedMoveClass;
var() int MaxJumps;
var() float JumpZModifer;
var PawnCRender RenderType;
var int JumpNum;
var struct OldRepeatMoveType
{
	var vector Accel;
	var eDodgeDir DMove;
	var bool bRun,bDuck,bFire,bAltFire,bJump;
	var int ViewRot;
} OldMoveDone;
var float ExactPing,LastPingSendTime;
var class<PlayerSpecies> Species;
var Texture ClientSkin,ReplicatedFlashTex;
var byte FireCounter,FireRate,MultiKills,ReplicatedFlashTime;
var float LastKillTime,LastClientSoundTime,GameEndedTime;
var() config float MouseAccelThreshold,MouseSmoothingMode,MouseSamplingTime;
var float SmoothedMouse[2], ZeroTime[2], SamplingTime[2], MaybeTime[2], OldSamples[4], MenuMousePos[2],
 SpaminCheck, LastSoundTimerr, LastCharacterChangeTime;
var int MouseSamples[2];
var() class<VoicePackMenu> VoicePackMenuClass;
var bool bIsNetOwner;
var bool bHasInitAudio,bAudioCheckOK;
var() config bool bPlayMP3Songs;
var() config float MP3SongVolume;
var ClientModifier ModifierList;
var vector OldCVelocity;
var VoicActor VoiceActor;
var texture ConstantOverlayTex;

var byte AnimAction;
var float AnimResetTime;

var UnrealSecurity SecurityHandler;

struct CompressedMoveType
{
	var byte PressedKeys;
	var bool bPressingWalk;
	var int ViewRot;
};
var VoicePackMenu OpenSpeechMenu;

// Adrenaline combo data
var transient int InputHistory[4];
var transient float LastKeyTime,NextAdrenCheckTime;
var transient int OldKey;
var int MinAdrenalineCost;
var transient Adrenaline ClientAdrenaline;
var() class<Combo> ComboList[16];
var Combo ActiveCombo;
const MinComboKeyTime = 0.05;   // keys pressed faster than this will be considered a double-press
const MaxComboKeyTime = 0.35;   // max time player has between button presses

var UTGRI MapGRI;

replication
{
	// Variables the server should send to the client.
	reliable if( Role==ROLE_Authority && !bNetOwner )
		bIsWalkingNow,bIsCrouchingNow,bPlayerFrozen,FireCounter,FireRate,ReplicatedFlashTex,ReplicatedFlashTime,AnimAction;
	reliable if( Role==ROLE_Authority )
		bIsTypingNow,ConstantOverlayTex;
	reliable if( Role==ROLE_Authority && bNetOwner )
		MaxJumps,JumpZModifer;
	unreliable if( Role==ROLE_AutonomousProxy )
		ShortServerMove,PreServerMove,ServerSpectateMove,ServerSpeech,ServerApplyCharacter;
	reliable if( Role<ROLE_Authority )
		SendClientPing,Join,Spectate,ServerBehindView,KillBots,Login,Logout,ServerDoCombo,ServerCustomCMD,
		ServerAllAmmo,ServerInvisible,ServerSetJumpZ,ServerSetSpeed,SendEvent,ServerGetWeapon,ServerPlayersOnly;
	unreliable if ( Role == ROLE_Authority && !bDemoRecording )
		ParseDeathMessage,ParseSpecialDMessage,ParseSuicideMessage,ClientBeginMMessage,ClientWaveMessage,SpreeMsg,ClientOpenSpeechMenu,
		ClientSetOverlayFlash,ReceiveLocalMessage,SwitchToMap,ShowSpeech,ClientOpenUWinMenu;
}

simulated function BeginPlay()
{
	if( Class'GameInfo'.Default.bCastShadow && Level.NetMode!=NM_DedicatedServer && Shadow==None )
		Shadow = Spawn(Class'PawnShadow',Self);
	Super.BeginPlay();
}
simulated function PostBeginPlay()
{
	if( Level.NetMode!=NM_DedicatedServer )
	{
		RenderType = Class'PawnCRender'.Static.AllocatePRender(RenderingClass,Self);
		RenderType.RenderOwner = Self;
	}
	Super.PostBeginPlay();
}
function SetMeshSkin( string InSkin, string InClass)
{
	if( InStr(InClass,".")==-1 )
		InClass = "UTDM."$InClass;
	UTPRI(PlayerReplicationInfo).StringSkin = InSkin;
	UTPRI(PlayerReplicationInfo).StringSpec = InClass;
	Species = Class<PlayerSpecies>(DynamicLoadObject(InClass,Class'Class'));
	if( Species==None )
		Species = Class'DeathMatchPlus'.Default.DefaultSpecies;
	bIsFemale = Species.Default.bIsFemale;
	UTPRI(PlayerReplicationInfo).LoadedSpecies = Species;
	Species.Static.GetOptionalMultiSkins(InSkin,PlayerReplicationInfo.Skin,PlayerReplicationInfo.Team,PlayerReplicationInfo.MultiSkins);
	UTPRI(PlayerReplicationInfo).SkinUdpCode++;
	if( UTPRI(PlayerReplicationInfo).SkinUdpCode>200 )
		UTPRI(PlayerReplicationInfo).SkinUdpCode = 1;
}
function ServerChangeSkin( coerce string SkinName, coerce string FaceName, byte TeamNum )
{
	if ( Level.Game.bCanChangeSkin )
		SetMeshSkin(SkinName,string(Species));
}
function BecomeCrouch()
{
	local vector V;
	
	if( bIsCrouchingNow ) Return;
	SetCollisionSize(Default.CollisionRadius*DrawScale,Default.CollisionHeight*DrawScale/2);
	bIsCrouchingNow = True;
	V.Z-=(Default.CollisionHeight*DrawScale/2);
	MoveSmooth(V);
	BaseEyeHeight = Default.BaseEyeHeight/2*DrawScale;
	EyeHeight = Default.CollisionHeight*DrawScale+Default.BaseEyeHeight*DrawScale;
}
function bool BecomeWalking()
{
	local vector V;
	
	if( !bIsCrouchingNow ) Return True;
	SetCollisionSize(Default.CollisionRadius*DrawScale,Default.CollisionHeight*DrawScale);
	bIsCrouchingNow = False;
	V.Z+=(Default.CollisionHeight*DrawScale/2);
	if( MoveSmooth(V) )
	{
		BaseEyeHeight = Default.BaseEyeHeight*DrawScale;
		EyeHeight = BaseEyeHeight-CollisionHeight;
		Return True;
	}
	else
	{
		SetCollisionSize(Default.CollisionRadius*DrawScale,Default.CollisionHeight*DrawScale/2);
		bIsCrouchingNow = True;
		Return False;
	}
}

// Tick adrenaline
function PlayerComboTick( float DeltaTime )
{
	local int CurrentKey, DiffKey;
	local int c, i;
	local bool bFullBuffer,bMatch;
	local Inventory In;

	if ( ViewPort(Player) == None )
		return;
	if( ClientAdrenaline==None || ClientAdrenaline.bDeleteMe || ClientAdrenaline.Owner!=Self )
	{
		if( NextAdrenCheckTime>Level.TimeSeconds )
			Return;
		For( In=Inventory; In!=None; In=In.Inventory )
		{
			if( Adrenaline(In)!=None )
			{
				ClientAdrenaline = Adrenaline(In);
				Break;
			}
		}
		if( ClientAdrenaline==None )
		{
			NextAdrenCheckTime = Level.TimeSeconds+2.f;
			Return;
		}
	}
	if( ClientAdrenaline.Charge<MinAdrenalineCost )
	{
		InputHistory[0] = 0;
		Return;
	}

	CurrentKey = 0;

	if (aBaseY > 0)		CurrentKey = CurrentKey | 1; // CK_Up
	else if (aBaseY < 0)  CurrentKey = CurrentKey | 2; // CK_Down
	if (aStrafe > 0)		CurrentKey = CurrentKey | 4; // CK_Left
	else if (aStrafe < 0)   CurrentKey = CurrentKey | 8; // CK_Right

	if ( CurrentKey != OldKey )
	{
		DiffKey = CurrentKey & ~OldKey; // keys that are pressed now but were not last frame

		if (DiffKey != 0)
		{
			if (Level.TimeSeconds - LastKeyTime < MinComboKeyTime) // if the time was too short between the last keypress, consider it a double press
			{
				InputHistory[0] = InputHistory[0] | DiffKey;
			}
			else if (Level.TimeSeconds - LastKeyTime > MaxComboKeyTime) // wipe the buffer if too long since last keypress
			{
				InputHistory[0] = Diffkey;
				InputHistory[1] = 0;
				LastKeyTime = Level.TimeSeconds;
			}
			else
			{
				bFullBuffer = true;
				for ( i=3; i>0; i-- )
				{
					InputHistory[i] = InputHistory[i - 1];
					if ( InputHistory[i] == 0 )
						bFullBuffer = false;
				}
				InputHistory[0] = DiffKey;
				LastKeyTime = Level.TimeSeconds;
			}

			// check for combo matches
			if ( bFullBuffer )
			{
				for ( c=0; c<16; c++ )
				{
					if ( ComboList[c] == None )
						break;
					bMatch = true;
					for ( i=0; i <4; i++)
						if ( InputHistory[i] != ComboList[c].Default.keys[i] )
						{
							bMatch = false;
							break;
						}

					if ( bMatch )
					{
						ServerDoCombo(ComboList[c]);
						InputHistory[0] = 0;
					}
				}
			}
		}

		OldKey = CurrentKey;
	}
}
function ServerDoCombo( Class<Combo> ComboMove )
{
	if( ComboMove==None || ActiveCombo!=None || ComboMove==Class'Combo' )
		Return;
	if( ClientAdrenaline==None )
	{
		ClientAdrenaline = Adrenaline(FindInventoryType(Class'Adrenaline'));
		if( ClientAdrenaline==None )
			Return;
	}
	if( ClientAdrenaline.Charge<ComboMove.Default.AdrenalineCost )
		Return;
	ActiveCombo = Spawn(ComboMove,Self);
	if( ActiveCombo==None )
		Return;
	ActiveCombo.InitilizeCombo();
}

function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation, Vector momentum, name damageType)
{
	if( Level.NetMode!=NM_Client )
		Super.TakeDamage(Damage,instigatedBy,hitlocation,momentum,damageType);
}
function Died(pawn Killer, name damageType, vector HitLocation)
{
	if( Level.NetMode!=NM_Client )
	{
		if( ActiveCombo!=None )
		{
			ActiveCombo.Destroy();
			ActiveCombo = None;
		}
		Super.Died(Killer,damageType,HitLocation);
	}
}
function WallLanded(vector HitNormal)
{
	if( bUpdatingMove ) Return;

	PlayLanded(Velocity.Z);
	if (Velocity.Z < -1.4 * JumpZ)
	{
		MakeNoise(-0.5 * Velocity.Z/(FMax(JumpZ, 150.0)));
		ShakeView(0.175 - 0.00007 * Velocity.Z, -0.85 * Velocity.Z, -0.002 * Velocity.Z); 
		if (Velocity.Z <= -1100)
		{
			if ( (Velocity.Z < -2000) && (ReducedDamageType != 'All') )
				TakeDamage(1000, None, Location, vect(0,0,0), 'fell');
			else if ( Role == ROLE_Authority )
				TakeDamage(-0.15 * (Velocity.Z + 1050), None, Location, vect(0,0,0), 'fell');
		}
	}
	else if ( (Level.Game != None) && (Level.Game.Difficulty > 1) && (Velocity.Z > 0.5 * JumpZ) )
		MakeNoise(0.1 * Level.Game.Difficulty);
}
simulated function PlayBeepSound()
{
	if( ViewTarget!=None )
		ViewTarget.PlaySound(sound'Beep',SLOT_Interface, 2.0);
	else PlaySound(sound'Beep',SLOT_Interface, 2.0);
}
function Landed(vector HitNormal)
{
	JumpNum = 0;
	if( bUpdatingMove ) Return;
	//Note - physics changes type to PHYS_Walking by default for landed pawns
	PlayLanded(Velocity.Z);
	if (Velocity.Z < -1.4 * JumpZ)
	{
		MakeNoise(-0.5 * Velocity.Z/(FMax(JumpZ, 150.0)));
		ShakeView(0.175 - 0.00007 * Velocity.Z, -0.85 * Velocity.Z, -0.002 * Velocity.Z);
		if( Velocity.Z <= -1100 && Role==ROLE_Authority )
		{
			if ( (Velocity.Z < -2000) && (ReducedDamageType != 'All') )
				TakeDamage(1000, None, Location, vect(0,0,0), 'fell');
			else TakeDamage(-0.15 * (Velocity.Z + 1050), None, Location, vect(0,0,0), 'fell');
		}
	}
	else if ( (Level.Game != None) && (Level.Game.Difficulty > 1) && (Velocity.Z > 0.5 * JumpZ) )
		MakeNoise(0.1 * Level.Game.Difficulty);				
	bJustLanded = true;
}
function PlayLanded(float impactVel)
{	
	impactVel = impactVel/JumpZ;
	impactVel = 0.1 * impactVel * impactVel;
	if( Species==None || bUpdatingMove ) Return;
	if ( impactVel > 0.17 )
		PlayOwnedSound(Species.Static.GetSound(S_LandGrunt), SLOT_Talk, FMin(5, 5 * impactVel),false,1200,FRand()*0.4+0.8);
	if ( !FootRegion.Zone.bWaterZone && (impactVel > 0.01) )
		PlayOwnedSound(Species.Static.GetSound(S_Land), SLOT_Interact, FClamp(4 * impactVel,0.5,5), false,1000, 1.0);
}
simulated function ClientOpenSpeechMenu()
{
	if( OpenSpeechMenu!=None )
	{
		if( OpenSpeechMenu.bHasDeleted )
		{
			OpenSpeechMenu.bHasDeleted = False;
			OpenSpeechMenu.PostBeginPlay();
		}
		else OpenSpeechMenu.CloseVoiceMenu();
	}
	else OpenSpeechMenu = Spawn(VoicePackMenuClass,Self);
}
function bool ValidWallDodge( eDodgeDir DodgeMove, vector X, vector Y, optional out vector HitNormal )
{
	local vector End,Hit;
	
	if (DodgeMove == DODGE_Forward)
		End = (100+CollisionRadius)*-X;
	else if (DodgeMove == DODGE_Back)
		End = (100+CollisionRadius)*X;
	else if (DodgeMove == DODGE_Left)
		End = (100+CollisionRadius)*-Y;
	else if (DodgeMove == DODGE_Right)
		End = (100+CollisionRadius)*Y;
	End+=Location;
	Return (Trace(Hit,HitNormal,End,Location,False)!=None);
}
exec function ViewPlayerNum(optional int num)
{
	local Pawn P;

	if ( num >= 0 )
	{
		P = Pawn(ViewTarget);
		if ( (P != None) && P.bIsPlayer && (P.PlayerReplicationInfo.TeamID == num) )
		{
			ViewTarget = None;
			bBehindView = false;
			return;
		}
		for ( P=Level.PawnList; P!=None; P=P.NextPawn )
			if ( P.bIsPlayer && (P.PlayerReplicationInfo.Team == PlayerReplicationInfo.Team)
				&& (P.PlayerReplicationInfo.TeamID == num) && PawnCRender(P)==None )
			{
				if ( P != self )
				{
					ViewTarget = P;
					bBehindView = true;
				}
				return;
			}
		return;
	}
	if ( Level.NetMode!=NM_Client )
	{
		ViewClass(class'Pawn');
		While ( (ViewTarget != None) 
				&& (!Pawn(ViewTarget).bIsPlayer || Pawn(ViewTarget).PlayerReplicationInfo.bIsSpectator) )
			ViewClass(class'Pawn', true);

		if ( ViewTarget != None )
			ClientMessage(ViewingFrom$ViewTarget.GetHumanName(), 'Event', true);
		else
			ClientMessage(ViewingFrom$OwnCamera, 'Event', true);
	}
}
state PlayerWalking
{
ignores SeePlayer, HearNoise, Bump, AnimEnd;

	exec function FeignDeath();
	
	function Dodge(eDodgeDir DodgeMove)
	{
		local vector X,Y,Z,HitNormal;

		if ( bIsCrouching )
			return;

		GetAxes(Rotation,X,Y,Z);
		if ( Physics==PHYS_Falling && !ValidWallDodge(DodgeMove,X,Y,HitNormal) )
			return;
		if( Physics==PHYS_Falling )
			WallLanded(HitNormal);
		if (DodgeMove == DODGE_Forward)
			Velocity = 1.5*GroundSpeed*X + (Velocity Dot Y)*Y;
		else if (DodgeMove == DODGE_Back)
			Velocity = -1.5*GroundSpeed*X + (Velocity Dot Y)*Y; 
		else if (DodgeMove == DODGE_Left)
			Velocity = 1.5*GroundSpeed*Y + (Velocity Dot X)*X; 
		else if (DodgeMove == DODGE_Right)
			Velocity = -1.5*GroundSpeed*Y + (Velocity Dot X)*X; 

		AnimAction = 4;
		AnimResetTime = Level.TimeSeconds+0.35;
		Velocity.Z = 160;
		if ( !bUpdatingMove && Species!=None )
			PlayOwnedSound(Species.Static.GetSound(S_Jump), SLOT_Talk, 1.0, true, 800, 1.0 );
		DodgeDir = DODGE_Active;
		if( Physics!=PHYS_Falling ) SetPhysics(PHYS_Falling);
	}
	
	function ProcessMove(float DeltaTime, vector NewAccel, eDodgeDir DodgeMove, rotator DeltaRot)	
	{
		local vector OldAccel;

		OldAccel = Acceleration;
		Acceleration = NewAccel;
		if ( (DodgeMove == DODGE_Active) && (Physics == PHYS_Falling) )
			DodgeDir = DODGE_Active;	
		else if ( (DodgeMove != DODGE_None) && (DodgeMove < DODGE_Active) )
			Dodge(DodgeMove);

		if ( bPressedJump )
			DoJump();
		if ( Physics==PHYS_Walking )
		{
			if (!bIsCrouching)
			{
				if (bDuck != 0)
				{
					bIsCrouching = true;
					BecomeCrouch();
				}
			}
			else if (bDuck == 0)
			{
				OldAccel = vect(0,0,0);
				bIsCrouching = !BecomeWalking();
			}
			bIsWalkingNow = (bRun!=0);
		}
		if( bIsCrouchingNow && bDuck==0 )
			Acceleration/=2;
	}
	function PlayerMove( float DeltaTime )
	{
		local vector X,Y,Z, NewAccel;
		local EDodgeDir OldDodge;
		local eDodgeDir DodgeMove;
		local rotator OldRotation;
		local float Speed2D;

		GetAxes(Rotation,X,Y,Z);

		aForward *= 0.4;
		aStrafe  *= 0.4;
		aLookup  *= 0.24;
		aTurn	*= 0.24;

		// Update acceleration.
		NewAccel = aForward*X + aStrafe*Y; 
		NewAccel.Z = 0;
		// Check for Dodge move
		if ( DodgeDir == DODGE_Active )
			DodgeMove = DODGE_Active;
		else
			DodgeMove = DODGE_None;
		if (DodgeClickTime > 0.0)
		{
			if ( DodgeDir < DODGE_Active )
			{
				OldDodge = DodgeDir;
				DodgeDir = DODGE_None;
				if (bEdgeForward && bWasForward)
					DodgeDir = DODGE_Forward;
				if (bEdgeBack && bWasBack)
					DodgeDir = DODGE_Back;
				if (bEdgeLeft && bWasLeft)
					DodgeDir = DODGE_Left;
				if (bEdgeRight && bWasRight)
					DodgeDir = DODGE_Right;
				if ( DodgeDir == DODGE_None)
					DodgeDir = OldDodge;
				else if ( DodgeDir != OldDodge )
					DodgeClickTimer = DodgeClickTime + 0.5 * DeltaTime;
				else 
					DodgeMove = DodgeDir;
			}
	
			if (DodgeDir == DODGE_Done)
			{
				DodgeClickTimer -= DeltaTime;
				if (DodgeClickTimer < -0.35) 
				{
					DodgeDir = DODGE_None;
					DodgeClickTimer = DodgeClickTime;
				}
			}		
			else if ((DodgeDir != DODGE_None) && (DodgeDir != DODGE_Active))
			{
				DodgeClickTimer -= DeltaTime;			
				if (DodgeClickTimer < 0)
				{
					DodgeDir = DODGE_None;
					DodgeClickTimer = DodgeClickTime;
				}
			}
		}
		if ( Physics == PHYS_Walking )
		{
			//if walking, look up/down stairs - unless player is rotating view
			if ( !bKeyboardLook && (bLook == 0) )
			{
				if ( bLookUpStairs )
					ViewRotation.Pitch = FindStairRotation(deltaTime);
				else if ( bCenterView )
				{
					ViewRotation.Pitch = ViewRotation.Pitch & 65535;
					if (ViewRotation.Pitch > 32768)
						ViewRotation.Pitch -= 65536;
					ViewRotation.Pitch = ViewRotation.Pitch * (1 - 12 * FMin(0.0833, deltaTime));
					if ( Abs(ViewRotation.Pitch) < 1000 )
						ViewRotation.Pitch = 0;	
				}
			}

			Speed2D = Sqrt(Velocity.X * Velocity.X + Velocity.Y * Velocity.Y);
			//add bobbing when walking
			if ( !bShowMenu )
			{
				if ( Speed2D < 10 )
					BobTime += 0.2 * DeltaTime;
				else
					BobTime += DeltaTime * (0.3 + 0.7 * Speed2D/GroundSpeed);
				WalkBob = Y * 0.65 * Bob * Speed2D * sin(6.0 * BobTime);
				if ( Speed2D < 10 )
					WalkBob.Z = Bob * 30 * sin(12 * BobTime);
				else
					WalkBob.Z = Bob * Speed2D * sin(12 * BobTime);
			}
		}	
		else if ( !bShowMenu )
		{ 
			BobTime = 0;
			WalkBob = WalkBob * (1 - FMin(1, 8 * deltatime));
		}

		// Update rotation.
		OldRotation = Rotation;
		UpdateRotation(DeltaTime, 1);

		if ( Role < ROLE_Authority ) // then save this move and replicate it
			ReplicateMove2(DeltaTime, NewAccel, DodgeMove, OldRotation - Rotation);
		else
			ProcessMove(DeltaTime, NewAccel, DodgeMove, OldRotation - Rotation);
		bPressedJump = False;
	}

	function BeginState()
	{
		WalkBob = vect(0,0,0);
		DodgeDir = DODGE_None;
		bIsCrouching = !BecomeWalking();
		bPressedJump = false;
		JumpNum = 0;
		if (Physics != PHYS_Falling) SetPhysics(PHYS_Walking);
	}
	
	function EndState()
	{
		WalkBob = vect(0,0,0);
		bIsCrouching = !BecomeWalking();
	}
}

function SavedMove GetFreeMove2()
{
	local SavedMove s;

	if ( FreeMoves == None )
		return Spawn(SavedMoveClass);
	else
	{
		s = FreeMoves;
		FreeMoves = FreeMoves.NextMove;
		s.NextMove = None;
		return s;
	}	
}
function ReplicateMove2( float DeltaTime, vector NewAccel, eDodgeDir DodgeMove, rotator DeltaRot )
{
	local SavedMove NewMove;
	local byte ClientRoll;
	local OldRepeatMoveType DT;

	// if am network client and am carrying flag - 
	//	make its position look good client side
	if ( (PlayerReplicationInfo != None) 
		&& (PlayerReplicationInfo.HasFlag != None) )
		PlayerReplicationInfo.HasFlag.FollowHolder(self);

	// Get a SavedMove actor to store the movement in.
	if ( SavedMoves == None )
	{
		SavedMoves = GetFreeMove2();
		NewMove = SavedMoves;
	}
	else
	{
		NewMove = SavedMoves;
		while ( NewMove.NextMove != None )
			NewMove = NewMove.NextMove;
		if ( NewMove.bSent )
		{
			NewMove.NextMove = GetFreeMove2();
			NewMove = NewMove.NextMove;
		}
	}

	NewMove.Delta = DeltaTime;
	if ( VSize(NewAccel) > 3072 )
		NewAccel = 3072 * Normal(NewAccel);
	UTSavedMove(NewMove).SaveMove(Self,NewAccel,bPressedJump,ViewRotation,DodgeMove,(bDuck > 0),(bRun > 0),(Velocity-OldCVelocity),DeltaTime);

	// Simulate the movement locally.
	ProcessMove(DeltaTime, NewAccel, DodgeMove, DeltaRot);
	AutonomousPhysics(DeltaTime);
	OldCVelocity = Velocity;
	//log("Role "$Role$" repmove at "$Level.TimeSeconds$" Move time "$100 * DeltaTime$" ("$Level.TimeDilation$")");

	// Send to the server
	NewMove.bSent = true;
	ClientRoll = (Rotation.Roll >> 8) & 255;
	DT.Accel = NewMove.Acceleration * 10;
	DT.DMove = NewMove.DodgeMove;
	DT.bRun = NewMove.bRun;
	DT.bDuck = NewMove.bDuck;
	DT.bJump = NewMove.bPressedJump;
	DT.bFire = (bJustFired || (bFire != 0));
	DT.bAltFire = (bJustAltFired || (bAltFire != 0));
	DT.ViewRot = (32767 & (ViewRotation.Pitch/2)) * 32768 + (32767 & (ViewRotation.Yaw/2));
	if( DT==OldMoveDone )
		ShortServerMove(NewMove.TimeStamp);
	else
	{
		OldMoveDone = DT;
		PreServerMove(NewMove.TimeStamp,DT.Accel,Location,DT.bRun,DT.bDuck,DT.bJump,DT.bFire,DT.bAltFire,DT.DMove, ClientRoll,DT.ViewRot);
	}
	bJustFired = false;
	bJustAltFired = false;
	//log("Replicated "$self$" stamp "$NewMove.TimeStamp$" location "$Location$" dodge "$NewMove.DodgeMove$" to "$DodgeDir);
}

// When repeating old move...
function ShortServerMove( float TimeStamp )
{
	ServerMove(TimeStamp,OldMoveDone.Accel,Location,OldMoveDone.bRun,OldMoveDone.bDuck,OldMoveDone.bJump,
	 OldMoveDone.bFire,OldMoveDone.bAltFire,OldMoveDone.DMove,0,OldMoveDone.ViewRot);
}
// Move to preform when spectating
function ServerSpectateMove( float ClientTime, vector Accel )
{
	PreServerMove(ClientTime,Accel,Location,False,False,False, False,False,DODGE_None,0,0);
}

function PreServerMove
(
	float TimeStamp, 
	vector InAccel, 
	vector ClientLoc,
	bool NewbRun,
	bool NewbDuck,
	bool NewbPressedJump, 
	bool bFired,
	bool bAltFired,
	eDodgeDir DodgeMove, 
	byte ClientRoll, 
	int View
)
{
	local OldRepeatMoveType DT;
	
	DT.Accel = InAccel;
	DT.DMove = DodgeMove;
	DT.bRun = NewbRun;
	DT.bDuck = NewbDuck;
	DT.bJump = NewbPressedJump;
	DT.bFire = bFired;
	DT.bAltFire = bAltFired;
	DT.ViewRot = View;
	OldMoveDone = DT;
	ServerMove(TimeStamp,InAccel,ClientLoc,NewbRun,NewbDuck,NewbPressedJump, bFired,bAltFired,DodgeMove,ClientRoll,View);
}
function ClientUpdatePosition()
{
	local SavedMove CurrentMove;
	local int realbRun, realbDuck;
	local bool bRealJump;

	bUpdatingMove = True;
	bUpdatePosition = false;
	realbRun= bRun;
	realbDuck = bDuck;
	bRealJump = bPressedJump;
	CurrentMove = SavedMoves;
	while ( CurrentMove != None )
	{
		if ( CurrentMove.TimeStamp <= CurrentTimeStamp )
		{
			SavedMoves = CurrentMove.NextMove;
			CurrentMove.NextMove = FreeMoves;
			FreeMoves = CurrentMove;
			UTSavedMove(FreeMoves).ClearMove();
			CurrentMove = SavedMoves;
		}
		else
		{
			UTSavedMove(CurrentMove).LoadMove(Self);
			MoveAutonomous(CurrentMove.Delta, CurrentMove.bRun, CurrentMove.bDuck, CurrentMove.bPressedJump, CurrentMove.DodgeMove, CurrentMove.Acceleration, rot(0,0,0));
			UTSavedMove(CurrentMove).PostLoadMove(Self);
			CurrentMove = CurrentMove.NextMove;
		}
	}
	OldCVelocity = Velocity;
	bDuck = realbDuck;
	bRun = realbRun;
	bPressedJump = bRealJump;
	bUpdatingMove = False;
}
function ClientAdjustPosition
(
	float TimeStamp, 
	name newState, 
	EPhysics newPhysics,
	float NewLocX, 
	float NewLocY, 
	float NewLocZ, 
	float NewVelX, 
	float NewVelY, 
	float NewVelZ,
	Actor NewBase
)
{
	local int SendPing;
	
	if ( CurrentTimeStamp > TimeStamp )
		return;
	ExactPing = Level.TimeSeconds-TimeStamp;
	if( PlayerReplicationInfo!=None )
	{
		PlayerReplicationInfo.Ping = int(ExactPing*1000);
		if( LastPingSendTime<Level.TimeSeconds )
		{
			SendPing = PlayerReplicationInfo.Ping/4;
			if( SendPing>255 )
				SendPing = 255;
			LastPingSendTime = Level.TimeSeconds+4;
			SendClientPing(SendPing);
		}
	}
	Super.ClientAdjustPosition(TimeStamp,newState,newPhysics,NewLocX,NewLocY,NewLocZ,NewVelX,NewVelY,NewVelZ,NewBase);
}
function SendClientPing( byte ThePing )
{
	if( PlayerReplicationInfo!=None )
		PlayerReplicationInfo.Ping = ThePing*4;
}
state PlayerSwimming
{
ignores SeePlayer, HearNoise, Bump,AnimEnd;

	function Landed(vector HitNormal)
	{
		//log(class$" Landed while swimming");
		if (Velocity.Z < -1.2 * JumpZ)
		{
			MakeNoise(-0.5 * Velocity.Z/(FMax(JumpZ, 150.0)));
			if (Velocity.Z <= -1100)
			{
				if ( (Velocity.Z < -2000) && (ReducedDamageType != 'All') )
					TakeDamage(1000, None, Location, vect(0,0,0), 'fell');
				else if ( Role == ROLE_Authority )
					TakeDamage(-0.1 * (Velocity.Z + 1050), Self, Location, vect(0,0,0), 'fell');
			}
		}
		else if ( (Level.Game != None) && (Level.Game.Difficulty > 1) && (Velocity.Z > 0.5 * JumpZ) )
			MakeNoise(0.1 * Level.Game.Difficulty);				
		bJustLanded = true;
		if ( Region.Zone.bWaterZone )
			SetPhysics(PHYS_Swimming);
		else
			GotoState('PlayerWalking');
	}
	
	function ZoneChange( ZoneInfo NewZone )
	{
		local actor HitActor;
		local vector HitLocation, HitNormal, checkpoint;

		if (!NewZone.bWaterZone)
		{
			SetPhysics(PHYS_Falling);
			if (bUpAndOut && CheckWaterJump(HitNormal)) //check for waterjump
			{
				velocity.Z = 330 + 2 * CollisionRadius; //set here so physics uses this for remainder of tick
				GotoState('PlayerWalking');
			}				
			else if (!FootRegion.Zone.bWaterZone || (Velocity.Z > 160) )
				GotoState('PlayerWalking');
			else //check if in deep water
			{
				checkpoint = Location;
				checkpoint.Z -= (CollisionHeight + 6.0);
				HitActor = Trace(HitLocation, HitNormal, checkpoint, Location, false);
				if (HitActor != None)
					GotoState('PlayerWalking');
				else
				{
					Enable('Timer');
					SetTimer(0.7,false);
				}
			}
			//log("Out of water");
		}
		else
		{
			Disable('Timer');
			SetPhysics(PHYS_Swimming);
		}
	}

	function ProcessMove(float DeltaTime, vector NewAccel, eDodgeDir DodgeMove, rotator DeltaRot)	
	{
		local vector X,Y,Z, Temp;
	
		GetAxes(ViewRotation,X,Y,Z);
		Acceleration = NewAccel;

		bUpAndOut = ((X Dot Acceleration) > 0) && ((Acceleration.Z > 0) || (ViewRotation.Pitch > 2048));
		if ( bUpAndOut && !Region.Zone.bWaterZone && CheckWaterJump(Temp) ) //check for waterjump
		{
			velocity.Z = 330 + 2 * CollisionRadius; //set here so physics uses this for remainder of tick
			GotoState('PlayerWalking');
		}				
	}
	function PlayerMove(float DeltaTime)
	{
		local rotator oldRotation;
		local vector X,Y,Z, NewAccel;
		local float Speed2D;
	
		GetAxes(ViewRotation,X,Y,Z);

		aForward *= 0.2;
		aStrafe  *= 0.1;
		aLookup  *= 0.24;
		aTurn	*= 0.24;
		aUp		 *= 0.1;  
		
		NewAccel = aForward*X + aStrafe*Y + aUp*vect(0,0,1); 
	
		//add bobbing when swimming
		if ( !bShowMenu )
		{
			Speed2D = Sqrt(Velocity.X * Velocity.X + Velocity.Y * Velocity.Y);
			WalkBob = Y * Bob *  0.5 * Speed2D * sin(4.0 * Level.TimeSeconds);
			WalkBob.Z = Bob * 1.5 * Speed2D * sin(8.0 * Level.TimeSeconds);
		}

		// Update rotation.
		oldRotation = Rotation;
		UpdateRotation(DeltaTime, 2);

		if ( Role < ROLE_Authority ) // then save this move and replicate it
			ReplicateMove2(DeltaTime, NewAccel, DODGE_None, OldRotation - Rotation);
		else
			ProcessMove(DeltaTime, NewAccel, DODGE_None, OldRotation - Rotation);
		bPressedJump = false;
	}

	function Timer()
	{
		if ( !Region.Zone.bWaterZone && (Role == ROLE_Authority) )
			GotoState('PlayerWalking');
		Disable('Timer');
	}
	
	function BeginState()
	{
		Disable('Timer');
	}
}

state PlayerFlying
{
ignores SeePlayer, HearNoise, Bump, AnimEnd;
		
	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z;

		GetAxes(Rotation,X,Y,Z);

		aForward *= 0.2;
		aStrafe  *= 0.2;
		aLookup  *= 0.24;
		aTurn	*= 0.24;
		aUp	  *= 0.1;  

		Acceleration = aForward*X + aStrafe*Y + aUp*Z;
		if( bPressedJump && aUp<=0.01 )
			bPressedJump = False;
		// Update rotation.
		UpdateRotation(DeltaTime, 2);

		if ( Role < ROLE_Authority ) // then save this move and replicate it
			ReplicateMove2(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
		else
			ProcessMove(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
	}
	function BeginState()
	{
		SetPhysics(PHYS_Flying);
	}
}

state CheatFlying
{
ignores SeePlayer, HearNoise, Bump, TakeDamage, AnimEnd;

	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z;

		GetAxes(ViewRotation,X,Y,Z);

		aForward *= 0.1;
		aStrafe  *= 0.1;
		aLookup  *= 0.24;
		aTurn	*= 0.24;
		aUp		 *= 0.1;
	
		Acceleration = aForward*X + aStrafe*Y + aUp*vect(0,0,1);  

		UpdateRotation(DeltaTime, 1);

		if ( Role < ROLE_Authority ) // then save this move and replicate it
			ReplicateMove2(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
		else
			ProcessMove(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
	}
	function BeginState()
	{
		EyeHeight = BaseEyeHeight;
		SetPhysics(PHYS_None);
	}
}

state PlayerWaiting
{
ignores FootZoneChange,HeadZoneChange,ClientOpenSpeechMenu,PlayerComboTick,ServerDoCombo;

	exec function Fire(optional float F)
	{
		bReadyToPlay = True;
	}
	exec function AltFire(optional float F)
	{
		bReadyToPlay = False;
	}
	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z;

		GetAxes(ViewRotation,X,Y,Z);

		aForward *= 0.1;
		aStrafe  *= 0.1;
		aLookup  *= 0.24;
		aTurn	*= 0.24;
		aUp		 *= 0.1;
	
		Acceleration = aForward*X + aStrafe*Y + aUp*vect(0,0,1);  

		UpdateRotation(DeltaTime, 1);

		if ( Role < ROLE_Authority ) // then save this move and replicate it
			ReplicateMove2(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
		else
			ProcessMove(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
	}

	function EndState()
	{
		SetCollision(true,true,true);
		SetCollisionSize(Default.CollisionRadius,Default.CollisionHeight);
		BaseEyeHeight = Default.BaseEyeHeight;
	}

	function BeginState()
	{
		bHidden = True;
		SetCollision(false,false,false);
		EyeHeight = BaseEyeHeight;
		SetPhysics(PHYS_None);
		SetCollisionSize(25,25);
		BaseEyeHeight = 0;
	}
}

exec function Join();
exec function Spectate()
{
	if( DeathMatchPlus(Level.Game).BecomeSpectator(Self) )
		GoToState('PlayerSpectating');
	else ClientMessage("You can't become a spectator this time, sorry.");
}

state PlayerSpectating
{
ignores ClientOpenSpeechMenu,FootZoneChange,HeadZoneChange,PlayerComboTick,ServerDoCombo;

	exec function AltFire( optional float F )
	{
		ViewSelf();
	}
	function ChangeTeam( int N )
	{
		Level.Game.ChangeTeam(self, N);
		Join();
	}
	exec function Fire( optional float F )
	{
		ViewPlayerNum(-1);
		BehindView(true);
	}
	exec function Spectate();
	exec function ShowSpeech();
	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z;

		GetAxes(ViewRotation,X,Y,Z);

		aForward *= 0.1;
		aStrafe  *= 0.1;
		aLookup  *= 0.24;
		aTurn	*= 0.24;
		aUp		 *= 0.1;
	
		Acceleration = aForward*X + aStrafe*Y + aUp*vect(0,0,1);  

		UpdateRotation(DeltaTime, 1);

		ProcessMove(DeltaTime, Acceleration, DODGE_None, rot(0,0,0));
		if ( Role < ROLE_Authority )
			ReplicateMove2(DeltaTime,Acceleration,DODGE_None,rot(0,0,0));
	}
	function EndState()
	{
		SetCollision(true,true,true);
		SetCollisionSize(Default.CollisionRadius,Default.CollisionHeight);
		BaseEyeHeight = Default.BaseEyeHeight;
	}
	function BeginState()
	{
		bHidden = True;
		SetCollision(false,false,false);
		EyeHeight = BaseEyeHeight;
		SetPhysics(PHYS_None);
		SetCollisionSize(25,25);
		BaseEyeHeight = 0;
		if( OpenSpeechMenu!=None )
			OpenSpeechMenu.CloseVoiceMenu();
		if( ActiveCombo!=None )
		{
			ActiveCombo.Destroy();
			ActiveCombo = None;
		}
	}
	exec function Join()
	{
		if( DeathMatchPlus(Level.Game).BecomeActivePlayer(Self) )
			GoToState('Dying');
		else ClientMessage("You can't become an active player this time, sorry.");
	}
	function ReplicateMove2( float DeltaTime, vector NewAccel, eDodgeDir DodgeMove, rotator DeltaRot )
	{
		local SavedMove NewMove;
		local byte ClientRoll;
		local OldRepeatMoveType DT;
	
		// if am network client and am carrying flag - 
		//	make its position look good client side
		if ( (PlayerReplicationInfo != None) 
			&& (PlayerReplicationInfo.HasFlag != None) )
			PlayerReplicationInfo.HasFlag.FollowHolder(self);
	
		// Get a SavedMove actor to store the movement in.
		if ( SavedMoves == None )
		{
			SavedMoves = GetFreeMove2();
			NewMove = SavedMoves;
		}
		else
		{
			NewMove = SavedMoves;
			while ( NewMove.NextMove != None )
				NewMove = NewMove.NextMove;
			if ( NewMove.bSent )
			{
				NewMove.NextMove = GetFreeMove2();
				NewMove = NewMove.NextMove;
			}
		}
	
		NewMove.Delta = DeltaTime;
		if ( VSize(NewAccel) > 3072 )
			NewAccel = 3072 * Normal(NewAccel);
		UTSavedMove(NewMove).SaveMove(Self,NewAccel,False,ViewRotation,DodgeMove,False,False,vect(0,0,0),DeltaTime);

		// Send to the server
		NewMove.bSent = true;
		ClientRoll = 0;
		DT.Accel = NewMove.Acceleration * 10;
		DT.DMove = NewMove.DodgeMove;
		DT.bRun = False;
		DT.bDuck = False;
		DT.bJump = False;
		DT.bFire = False;
		DT.bAltFire = False;
		DT.ViewRot = 0;
		if( DT==OldMoveDone )
			ShortServerMove(NewMove.TimeStamp);
		else
		{
			OldMoveDone = DT;
			ServerSpectateMove(NewMove.TimeStamp,DT.Accel);
		}
		bJustFired = false;
		bJustAltFired = false;
		//log("Replicated "$self$" stamp "$NewMove.TimeStamp$" location "$Location$" dodge "$NewMove.DodgeMove$" to "$DodgeDir);
	}
}
state Dying
{
ignores SeePlayer, HearNoise, KilledBy, Bump, HitWall, HeadZoneChange, FootZoneChange, ZoneChange, SwitchWeapon,
 Falling, PainTimer, PlayerComboTick, ServerDoCombo;

	function PlayerCalcView(out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
	{
		Global.PlayerCalcView(ViewActor,CameraLocation,CameraRotation);
	}
	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z;

		if ( !bFrozen )
		{
			if ( bPressedJump )
				Fire(0);
			GetAxes(ViewRotation,X,Y,Z);
			// Update view rotation.
			aLookup  *= 0.24;
			aTurn	*= 0.24;
			ViewRotation.Yaw += 32.0 * DeltaTime * aTurn;
			ViewRotation.Pitch += 32.0 * DeltaTime * aLookUp;
			ViewRotation.Pitch = ViewRotation.Pitch & 65535;
			If ((ViewRotation.Pitch > 18000) && (ViewRotation.Pitch < 49152))
			{
				If (aLookUp > 0) 
					ViewRotation.Pitch = 18000;
				else
					ViewRotation.Pitch = 49152;
			}
			if ( Role < ROLE_Authority ) // then save this move and replicate it
				ReplicateMove2(DeltaTime, vect(0,0,0), DODGE_None, rot(0,0,0));
			bPressedJump = false;
		}
		ViewShake(DeltaTime);
		ViewFlash(DeltaTime);
	}
	function ServerReStartPlayer()
	{
		RealReStartPlayer();
	}
}

state GameEnded
{
ignores SeePlayer, HearNoise, KilledBy, Bump, HitWall, HeadZoneChange, FootZoneChange, ZoneChange,
 Falling, TakeDamage, PainTimer, Died, Suicide, PlayerComboTick, ServerDoCombo;

	exec function AltFire( optional float F )
	{
		Fire(F);
	}
	exec function Fire( optional float F )
	{
		if( Level.NetMode==NM_StandAlone && GameEndedTime<Level.TimeSeconds )
			Level.Game.RestartGame();
	}
	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z;
		
		GetAxes(ViewRotation,X,Y,Z);
		// Update view rotation.

		if ( !bFixedCamera )
		{
			aLookup  *= 0.24;
			aTurn	*= 0.24;
			ViewRotation.Yaw += 32.0 * DeltaTime * aTurn;
			ViewRotation.Pitch += 32.0 * DeltaTime * aLookUp;
			ViewRotation.Pitch = ViewRotation.Pitch & 65535;
			If ((ViewRotation.Pitch > 18000) && (ViewRotation.Pitch < 49152))
			{
				If (aLookUp > 0) 
					ViewRotation.Pitch = 18000;
				else
					ViewRotation.Pitch = 49152;
			}
		}
		ViewShake(DeltaTime);
		ViewFlash(DeltaTime);

		if ( Role < ROLE_Authority ) // then save this move and replicate it
			ReplicateMove2(DeltaTime, vect(0,0,0), DODGE_None, rot(0,0,0));
		else
			ProcessMove(DeltaTime, vect(0,0,0), DODGE_None, rot(0,0,0));
		bPressedJump = false;
	}
	function BeginState()
	{
		EndZoom();
		AnimRate = 0.0;
		bFire = 0;
		bAltFire = 0;
		SetCollision(false,false,false);
		bShowScores = true;
		bFrozen = true;
		GameEndedTime = Level.TimeSeconds+5;
		if ( !bFixedCamera )
		{
			FindGoodView();
			bBehindView = true;
		}
		SetTimer(1.5, false);
		SetPhysics(PHYS_None);
		Velocity = vect(0,0,0);
		bPlayerFrozen = True;
		if( ActiveCombo!=None )
		{
			ActiveCombo.Destroy();
			ActiveCombo = None;
		}
	}
	function EndState()
	{
		EndZoom();
		SetCollision(true,true,true);
		bShowScores = false;
		bFrozen = false;
		bBehindView = false;
		bPlayerFrozen = false;
	}
}
function DoJump( optional float F )
{
	if ( CarriedDecoration != None )
		return;
	if ( !bIsCrouching && (Physics == PHYS_Walking) )
	{
		if ( !bUpdatingMove && Species!=None )
			PlayOwnedSound(Species.Static.GetSound(S_Jump), SLOT_Talk, 1.5, true, 1200, 1.0 );
		if ( (Level.Game != None) && (Level.Game.Difficulty > 0) )
			MakeNoise(0.1 * Level.Game.Difficulty);
		if ( bCountJumps && (Role == ROLE_Authority) )
			Inventory.OwnerJumped();
		Velocity.Z = JumpZ;
		if ( Base != Level )
			Velocity.Z += Base.Velocity.Z; 
		SetPhysics(PHYS_Falling);
	}
	else if( Physics==PHYS_Falling && JumpNum<MaxJumps && Abs(Velocity.Z)<100 )
	{
		JumpNum++;
		if ( !bUpdatingMove && Species!=None )
			PlayOwnedSound(Species.Static.GetSound(S_Jump), SLOT_Talk, 1.5, true, 1200, 1.0 );
		if ( (Level.Game != None) && (Level.Game.Difficulty > 0) )
			MakeNoise(0.1 * Level.Game.Difficulty);
		Velocity.Z = Default.JumpZ+JumpZModifer;
	}
}
simulated function Destroyed()
{
	if( Level.NetMode!=NM_Client && !bIsNetOwner && Health>0 && Inventory!=None )
		KilledBy(Self);
	if( RenderType!=None && !bIsNetOwner && !RenderType.bDeleteMe )
		RenderType.FreePRender();
	if( Level.NetMode!=NM_Client && SecurityHandler!=None && !bIsNetOwner )
		SecurityHandler.Destroy();
	Super.Destroyed();
}
simulated function PlayOwnedSound( sound Sound, optional ESoundSlot Slot, optional float	Volume,
	optional bool bNoOverride, optional float Radius, optional float Pitch )
{
	local bool bRealPL;
	
	if( Volume==0 )
		Volume = TransientSoundVolume;
	if( Radius==0 )
	{
		Radius = TransientSoundRadius;
		if( Radius==0 )
			Radius = 1200;
	}
	if( Pitch==0 )
		Pitch = 1;
	if( Level.NetMode==NM_Client )
		PlaySound(Sound,Slot,Volume,bNoOverride,Radius,Pitch);
	else
	{
		bRealPL = bIsPlayer;
		if( Player==None || Player.Console==None )
			bIsPlayer = False;
		ReplicatedSound(Sound,Slot,Volume,bNoOverride,Radius,Pitch);
		bIsPlayer = bRealPL;
	}
}
function ReplicatedSound( sound Sound, ESoundSlot Slot, float	Volume, bool bNoOverride, float Radius, float Pitch )
{
	PlaySound(Sound,Slot,Volume,bNoOverride,Radius,Pitch);
}
function ParseDeathMessage( name DamageType, optional PlayerReplicationInfo Killer, optional string KillerS,
 optional PlayerReplicationInfo Victim, optional string VictimS, optional class<Weapon> KillerGun )
{
	local string Msg,GunName;
	local UTGRI G;
	local DmgTypeModifier DT;

	if( Viewport(Player)==None )
		Return;
	if( Killer!=None )
		KillerS = Killer.PlayerName;
	if( Victim!=None )
		VictimS = Victim.PlayerName;
	if( KillerS=="" )
		KillerS = "somebody";
	if( VictimS=="" )
		VictimS = "somebody";
	if( Victim!=None && Victim==PlayerReplicationInfo )
		ClientMessage("You were killed by"@KillerS,'KillByMsg');
	else if( Killer!=None && Killer==PlayerReplicationInfo )
		ClientMessage("You killed"@VictimS,'KilledMsg');
	G = Class'DeathMatchPlus'.Static.GetMapGRI(Self);
	if( G!=None && G.DmgModifiersList!=None )
		DT = G.DmgModifiersList.ModifyDamageType(DamageType);
	if( DT!=None )
		Msg = DT.GetDeathMessage(DamageType,KillerS,VictimS,Killer,Victim,KillerGun);
	else if( KillerGun!=None && DamageType!='' && (DamageType==KillerGun.Default.MyDamageType ||
	 DamageType==KillerGun.Default.AltDamageType) )
	{
		GunName = KillerGun.Default.ItemName;
		if( GunName=="" )
			GunName = GetItemName(string(KillerGun));
		Msg = KillerGun.Default.DeathMessage;
		Msg = Class'DeathMatchPlus'.Static.ReplaceText(Msg,"%k",KillerS,True);
		Msg = Class'DeathMatchPlus'.Static.ReplaceText(Msg,"%o",VictimS,True);
		Msg = Class'DeathMatchPlus'.Static.ReplaceText(Msg,"%w",GunName,True);
	}
	else if( DamageType=='LavaPushed' )
		Msg = KillerS@"made"@VictimS@"take a deadly swim.";
	else if( DamageType=='FellPushed' )
		Msg = KillerS@"pushed"@VictimS@"over edge.";
	else if( DamageType=='SlimePushed' )
		Msg = KillerS@"gave"@VictimS@"extreme slimy feeling.";
	else if( DamageType=='DrownPushed' )
		Msg = KillerS@"made"@VictimS@"drown.";
	else
	{
		if( DamageType=='hacked' )
			Msg = Class'UnrealGameInfo'.Default.HackedMessage;
		else if( DamageType=='corroded' )
			Msg = Class'UnrealGameInfo'.Default.CorrodedMessage;
		else if( DamageType=='burned' )
			Msg = Class'UnrealGameInfo'.Default.BurnedMessage;
		else if( DamageType=='exploded' )
			Msg = Class'UnrealGameInfo'.Default.ExplodeMessage;
		else if( DamageType=='decapited' || DamageType=='headshot' )
			Msg = Class'UnrealGameInfo'.Default.DeathVerb$Class'UnrealGameInfo'.Default.HeadLossMessage[Rand(2)];
		else
		{
			if( FRand()<0.3 )
				Msg = Class'UnrealGameInfo'.Default.MajorDeathMessage[Rand(3)];
			else
			{
				if( FRand()<0.3 )
					Msg = Class'UnrealGameInfo'.Default.DeathModifier[Rand(5)];
				Msg = Msg$Class'UnrealGameInfo'.Default.DeathMessage[Rand(32)];
			}
			Msg = Class'UnrealGameInfo'.Default.DeathVerb$Msg;
		}
		Msg = VictimS$Msg$Class'UnrealGameInfo'.Default.DeathPrep$KillerS;
	}
	ClientMessage(Msg,'DeathMessage');
}
function ParseSpecialDMessage( string Msg, optional PlayerReplicationInfo Killer, optional string KillerS,
 optional PlayerReplicationInfo Victim, optional string VictimS, optional class<Weapon> KillerGun )
{
	local string GunName;

	if( Viewport(Player)==None )
		Return;
	if( Killer!=None )
		KillerS = Killer.PlayerName;
	if( Victim!=None )
		VictimS = Victim.PlayerName;
	if( KillerS=="" )
		KillerS = "somebody";
	if( VictimS=="" )
		VictimS = "somebody";
	if( Victim!=None && Victim==PlayerReplicationInfo )
		ClientMessage("You were killed by"@KillerS,'KillByMsg');
	else if( Killer!=None && Killer==PlayerReplicationInfo )
		ClientMessage("You killed"@VictimS,'KilledMsg');
	Msg = Class'DeathMatchPlus'.Static.ReplaceText(Msg,"%k",KillerS,True);
	Msg = Class'DeathMatchPlus'.Static.ReplaceText(Msg,"%o",VictimS,True);
	if( KillerGun==None )
		GunName = "a Weapon";
	else
	{
		GunName = KillerGun.Default.ItemName;
		if( GunName=="" )
			GunName = GetItemName(string(KillerGun));
	}
	Msg = Class'DeathMatchPlus'.Static.ReplaceText(Msg,"%w",GunName,True);
	ClientMessage(Msg,'DeathMessage');
}
function ParseSuicideMessage( name DamageType, optional PlayerReplicationInfo Killer, optional string KillerS )
{
	local string Msg;
	local UTGRI G;
	local DmgTypeModifier DT;

	if( Viewport(Player)==None )
		Return;
	if( Killer!=None )
		KillerS = Killer.PlayerName;
	if( KillerS=="" )
		KillerS = "somebody";

	G = Class'DeathMatchPlus'.Static.GetMapGRI(Self);
	if( G!=None && G.DmgModifiersList!=None )
		DT = G.DmgModifiersList.ModifyDamageType(DamageType);
	if( DT!=None )
		Msg = DT.GetSuicideMessage(DamageType,KillerS,Killer);
	else
	{
		if( DamageType=='hacked' )
			Msg = Class'UnrealGameInfo'.Default.HackedMessage;
		else if( DamageType=='corroded' )
			Msg = Class'UnrealGameInfo'.Default.CorrodedMessage;
		else if( DamageType=='burned' )
			Msg = Class'UnrealGameInfo'.Default.BurnedMessage;
		else if( DamageType=='exploded' )
			Msg = Class'UnrealGameInfo'.Default.ExplodeMessage;
		else if( DamageType=='suicided' )
			Msg = Class'UnrealGameInfo'.Default.SuicideMessage;
		else if( DamageType=='drowned' )
			Msg = Class'UnrealGameInfo'.Default.DrownedMessage;
		else if( DamageType=='fell' )
			Msg = Class'UnrealGameInfo'.Default.FallMessage;
		else if( DamageType=='SpaceWalk' )
			Msg = " forgot they are in outer space.";
		else if( DamageType=='Frozen' )
			Msg = " froze to their death.";
		else if( DamageType=='decapited' || DamageType=='headshot' )
			Msg = Class'UnrealGameInfo'.Default.DeathVerb$Class'UnrealGameInfo'.Default.HeadLossMessage[Rand(2)];
		else
		{
			if( FRand()<0.3 )
				Msg = Class'UnrealGameInfo'.Default.MajorDeathMessage[Rand(3)];
			else
			{
				if( FRand()<0.3 )
					Msg = Class'UnrealGameInfo'.Default.DeathModifier[Rand(5)];
				Msg = Msg$Class'UnrealGameInfo'.Default.DeathMessage[Rand(32)];
			}
			Msg = Class'UnrealGameInfo'.Default.DeathVerb$Msg;
		}
		Msg = KillerS$Msg;
	}
	ClientMessage(Msg,'DeathMessage');
}
function PlayHit(float Damage, vector HitLocation, name damageType, float MomentumZ)
{
	local float rnd;
	local Bubble1 bub;
	local vector BloodOffset;
	local UTGRI GG;
	local DmgTypeModifier DG;
	local texture MM;

	if ( (Damage <= 0) && (ReducedDamageType != 'All') )
		return;

	//DamageClass = class(damageType);
	if ( ReducedDamageType != 'All' ) //spawn some blood
	{
		if (damageType == 'Drowned')
		{
			bub = spawn(class 'Bubble1',,, Location 
				+ 0.7 * CollisionRadius * vector(ViewRotation) + 0.3 * EyeHeight * vect(0,0,1));
			if (bub != None)
				bub.DrawScale = FRand()*0.06+0.04; 
		}
		else if ( (damageType != 'Burned') && (damageType != 'Corroded') 
					&& (damageType != 'Fell') )
		{
			BloodOffset = 0.2 * CollisionRadius * Normal(HitLocation - Location);
			BloodOffset.Z = BloodOffset.Z * 0.5;
			if ( (Level.Game != None) && Level.Game.bVeryLowGore )
				spawn(class 'BloodBurst',self,,hitLocation + BloodOffset, rotator(BloodOffset));
			else
				spawn(class 'BloodSpray',self,,hitLocation + BloodOffset, rotator(BloodOffset));
		}
		GG = Class'DeathMatchPlus'.Static.GetMapGRI(Self);
		if( GG!=None && GG.DmgModifiersList!=None )
			DG = GG.DmgModifiersList.ModifyDamageType(damageType);
		if( DG!=None )
			MM = DG.GetDamageOverlay(damageType,rnd);
		if( MM!=None )
			FlashOverlayMaterial(MM,rnd,True);
		else if( damageType=='Corroded' )
			FlashOverlayMaterial(Texture'fireeffect15',0.2+FRand()*0.45,True);
		else if( damageType=='Burned' )
			FlashOverlayMaterial(Texture'FireEffect28a',0.2+FRand()*0.45,True);
		else if( damageType=='Jolted' )
			FlashOverlayMaterial(Texture'FireEffect3a',0.2+FRand()*0.45,True);
	}	

	rnd = FClamp(Damage, 20, 60);
	if ( damageType == 'Burned' )
		ClientFlash( -0.009375 * rnd, rnd * vect(16.41, 11.719, 4.6875));
	else if ( damageType == 'corroded' )
		ClientFlash( -0.01171875 * rnd, rnd * vect(9.375, 14.0625, 4.6875));
	else if ( damageType == 'Drowned' )
		ClientFlash(-0.390, vect(312.5,468.75,468.75));
	else 
		ClientFlash( -0.019 * rnd, rnd * vect(26.5, 4.5, 4.5));

	ShakeView(0.15 + 0.005 * Damage, Damage * 30, 0.3 * Damage); 
	PlayTakeHitSound(Damage, damageType, 1);
	PlayTakeHit(0.1, hitLocation, Damage);
}
function PlayTakeHitSound(int damage, name damageType, int Mult)
{
	if ( Level.TimeSeconds<LastPainSound || Species==None )
		return;
	LastPainSound = Level.TimeSeconds+0.3;

	if ( HeadRegion.Zone.bWaterZone )
	{
		if ( damageType == 'Drowned' )
			PlaySound(Species.Static.GetSound(S_DPain), SLOT_Pain, 1.5);
		else PlaySound(Species.Static.GetSound(S_UnderWater), SLOT_Pain,2.0,,,Frand()*0.15+0.9);
	}
	else PlaySound(Species.Static.GetSound(S_Pain), SLOT_Pain,2.0,,,Frand()*0.15+0.9);		
}
function Carcass SpawnCarcass();
function SpawnClientCarcass( name DamageType, vector HitLoc, vector Momentum )
{
	local PawnCarcassMark PC;
	
	if( Species!=None )
	{
		if( DamageType=='' )
			DamageType = 'DamageType';
		PC = Spawn(Class'PawnCarcassMark');
		PC.ThePlayer = UTPRI(PlayerReplicationInfo);
		PC.ColH = CollisionHeight;
		PC.ColR = CollisionRadius;
		PC.DeathDamageType = DamageType;
		PC.HitLocation = (HitLoc-Location);
		PC.Velocity = Momentum;
		PC.DrawScale = DrawScale;
		if( Level.NetMode!=NM_DedicatedServer )
			PC.PostNetReceive();
	}
}
function PlayDying(name DamageType, vector HitLoc)
{
	if( Level.NetMode==NM_Client ) Return;
	PlayDyingSound();
	SpawnClientCarcass(DamageType,HitLoc,Velocity+vect(0,0,10));
}
function PlayDeathHit(float Damage, vector HitLocation, name damageType)
{
	local Bubble1 bub;

	if ( Region.Zone.bDestructive && (Region.Zone.ExitActor != None) )
		Spawn(Region.Zone.ExitActor);
	if (HeadRegion.Zone.bWaterZone)
	{
		bub = spawn(class 'Bubble1',,, Location 
			+ 0.3 * CollisionRadius * vector(Rotation) + 0.8 * EyeHeight * vect(0,0,1));
		if (bub != None)
			bub.DrawScale = FRand()*0.08+0.03; 
		bub = spawn(class 'Bubble1',,, Location 
			+ 0.2 * CollisionRadius * VRand() + 0.7 * EyeHeight * vect(0,0,1));
		if (bub != None)
			bub.DrawScale = FRand()*0.08+0.03; 
		bub = spawn(class 'Bubble1',,, Location 
			+ 0.3 * CollisionRadius * VRand() + 0.6 * EyeHeight * vect(0,0,1));
		if (bub != None)
			bub.DrawScale = FRand()*0.08+0.03; 
	}

	if ( (damageType != 'Drowned') && (damageType != 'Corroded') )
		spawn(class 'BloodSpray',self,'', hitLocation);
}
function PlayDyingSound()
{
	if( Species!=None )
	{
		if( HeadRegion.Zone.bWaterZone )
			PlaySound(Species.Static.GetSound(S_UWDeath), SLOT_Pain,,,,Frand()*0.2+0.9);
		else PlaySound(Species.Static.GetSound(S_Death), SLOT_Talk);
	}
}
function PlayGutHit(float tweentime);

simulated function PlayRecoil(float Rate)
{
	local int R;
	
	R = Rate*15;
	if( R>255 )
		R = 255;
	FireRate = R;
	FireCounter++;
	if( FireCounter>250 )
		FireCounter = 1;
}
simulated function FlashOverlayMaterial( Texture Tex, float FlashTime, bool bReplicateForOwner )
{
	local int R;
	
	if( bHidden ) Return;
	R = FlashTime*15;
	if( R>255 )
		R = 255;
	ReplicatedFlashTime = R;
	ReplicatedFlashTex = Tex;
	if( bReplicateForOwner )
		ClientSetOverlayFlash(Tex,R);
}
function ClientSetOverlayFlash( Texture T, byte FlashTime )
{
	ClientOverlayFlash(T,FlashTime);
}
simulated function ClientOverlayFlash( Texture Tex, byte FlashTime )
{
	local OverlayMaterialA MA;
	
	if( RenderType==None || bHidden ) Return;
	MA = Spawn(class'OverlayMaterialA',RenderType);
	MA.DrawScale = RenderType.DrawScale;
	MA.Fatness = RenderType.Fatness+2;
	MA.Mesh = RenderType.Mesh;
	MA.Texture = Tex;
	MA.LifeSpan = float(FlashTime)/5;
	if( MA.LifeSpan==0 )
		MA.LifeSpan = 0.025;
	MA.StartUpLS = MA.LifeSpan;
}
simulated function bool AdjustHitLocation(out vector HitLocation, vector TraceDir)
{
	Return True; // Always hit, becouse we use collisionsize on crouch!
}
function Typing( bool bTyping )
{
	bIsTyping = bTyping;
	bIsTypingNow = bTyping;
}
function PlayerCalcView(out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	Super.PlayerCalcView(ViewActor,CameraLocation,CameraRotation);
	if( AnimAction>0 && Level.TimeSeconds>AnimResetTime )
		AnimAction = 0;
	if( VoiceActor!=None )
	{
		VoiceActor.SetLocation(CameraLocation);
		VoiceActor.SetLocation(CameraLocation);
	}
	if( !bBehindView && (ViewActor==None || ViewActor==Self) && RenderType!=None )
		RenderType.LastRenderedTime = Level.TimeSeconds;
}
function CalcBehindView(out vector CameraLocation, out rotator CameraRotation, float Dist)
{
	local vector View,HitLocation,HitNormal;
	local float ViewDist;
	local Actor A;

	A = ViewTarget;
	if( A==None )
		A = Self;
	Dist = A.CollisionRadius*15;
	if( Dist<200 )
		Dist = 200;
	CameraRotation = ViewRotation;
	View = vect(1,0,0) >> CameraRotation;
	if( Trace( HitLocation, HitNormal, CameraLocation - (Dist + 30) * vector(CameraRotation), CameraLocation ) != None )
		ViewDist = FMin( (CameraLocation - HitLocation) Dot View, Dist );
	else
		ViewDist = Dist;
	CameraLocation -= (ViewDist - 30) * View; 
}
exec function BehindView( Bool B )
{
	ServerBehindView(B);
}

function ServerBehindView( bool B )
{
	bBehindView = B;
}
exec function FunctionKey( byte Num )
{
	if( Num==4 )
		ServerBehindView(!bBehindView);
}

function ClientBeginMMessage( byte Calling )
{
	if( Calling==0 )
		ClientMessage("Waiting for players...",'MatchMsg');
	else if( Calling==1 )
		ClientMessage("Hit [Fire] to begin the match, you are NOT ready.",'MatchMsg');
	else if( Calling==2 )
		ClientMessage("Waiting for other players to be ready.",'MatchMsg');
	else if( Calling==3 )
		ClientMessage("Hit [Fire] to join the match...",'MatchMsg');
	else if( Calling==4 )
	{
		ClientMessage("Play",'MatchMsg');
		ClientPlaySound(Sound'Play');
	}
	else if( Calling==5 )
	{
		ClientMessage("Overtime!",'SpreeMsg');
		ClientPlaySound(Sound'UT2K3Fanfare01');
	}
	else
	{
		ClientMessage("Match is about to begin..."@(Calling-5),'MatchMsg');
		ClientPlaySound(Sound'selectJ');
	}
}
// For invasion...
function ClientWaveMessage( byte Calling )
{
	if( Calling==0 )
	{
		ClientMessage("Next wave in...",'WaveMsg');
		ClientPlaySound(Sound'next_wave_in');
	}
	else
	{
		ClientMessage(Calling@"...",'WaveMsg');
		if( Calling==1 )
			ClientPlaySound(Sound'one');
		if( Calling==2 )
			ClientPlaySound(Sound'two');
		if( Calling==3 )
			ClientPlaySound(Sound'three');
		if( Calling==4 )
			ClientPlaySound(Sound'four');
		if( Calling==5 )
			ClientPlaySound(Sound'five');
		if( Calling==6 )
			ClientPlaySound(Sound'six');
		if( Calling==7 )
			ClientPlaySound(Sound'seven');
		if( Calling==8 )
			ClientPlaySound(Sound'eight');
		if( Calling==9 )
			ClientPlaySound(Sound'nine');
		if( Calling==10 )
			ClientPlaySound(Sound'ten');
	}
}
function SpreeMsg( byte Num, optional string A, optional string B, optional bool bItsMe )
{
	if( A=="" )
	{
		if( Num==0 )
		{
			ClientMessage("Double Kill!",'SpreeMsg');
			ClientPlaySound(Sound'Double_Kill');
		}
		else if( Num==1 )
		{
			ClientMessage("Multi Kill!",'SpreeMsg');
			ClientPlaySound(Sound'MultiKill');
		}
		else if( Num==2 )
		{
			ClientMessage("Ultra Kill!",'SpreeMsg');
			ClientPlaySound(Sound'UltraKill');
		}
		else if( Num==3 )
		{
			ClientMessage("M-M-M-MONSTER KILL!",'SpreeMsg');
			ClientPlaySound(Sound'MonsterKill_F');
		}
		else if( Num==4 )
		{
			ClientMessage("L-LUDICROUS KILL!",'SpreeMsg');
			ClientPlaySound(Sound'LudicrousKill_F');
		}
		else
		{
			ClientMessage("HOLY SHIT!",'SpreeMsg');
			ClientPlaySound(Sound'HolyShit_F');
		}
	}
	else if( Num==0 )
	{
		if( B=="" )
		{
			if( bItsMe )
				ClientMessage(A@"was looking good until she killed herself!",'SpreeMsg');
			else ClientMessage(A@"was looking good until he killed himself!",'SpreeMsg');
		}
		else ClientMessage(A$"'s Killing Spree was ended by"@B,'SpreeMsg');
	}
	else if( Num==1 )
	{
		if( bItsMe )
		{
			ClientMessage("Killing Spree!",'SpreeMsg');
			ClientPlaySound(Sound'Killing_Spree');
		}
		else ClientMessage(A@"is on a killing spree!",'SpreeMsg');
	}
	else if( Num==2 )
	{
		if( bItsMe )
		{
			ClientMessage("Rampage!",'SpreeMsg');
			ClientPlaySound(Sound'Rampage');
		}
		else ClientMessage(A@"is on a rampage!",'SpreeMsg');
	}
	else if( Num==3 )
	{
		if( bItsMe )
		{
			ClientMessage("Dominating!",'SpreeMsg');
			ClientPlaySound(Sound'Dominating');
		}
		else ClientMessage(A@"is dominating!",'SpreeMsg');
	}
	else if( Num==4 )
	{
		if( bItsMe )
		{
			ClientMessage("Unstoppable!",'SpreeMsg');
			ClientPlaySound(Sound'Unstoppable');
		}
		else ClientMessage(A@"is unstoppable!",'SpreeMsg');
	}
	else if( Num==5 )
	{
		if( bItsMe )
		{
			ClientMessage("GODLIKE!",'SpreeMsg');
			ClientPlaySound(Sound'GodLike');
		}
		else ClientMessage(A@"is Godlike!",'SpreeMsg');
	}
	else if( Num==6 )
	{
		if( bItsMe )
		{
			ClientMessage("Wicked SICK!",'SpreeMsg');
			ClientPlaySound(Sound'WhickedSick');
		}
		else ClientMessage(A@"is Wicked SICK!",'SpreeMsg');
	}
	else if( Num==7 )
	{
		if( bItsMe )
		{
			ClientMessage("First Blood!",'FBMsg');
			ClientPlaySound(Sound'first_blood');
		}
		else ClientMessage(A@"drew first blood!",'FBMsg');
	}
	else if( Num==8 )
	{
		if( bItsMe )
			ClientMessage("You are out!",'SpreeMsg');
		else ClientMessage(A@"is out!",'SpreeMsg');
	}
}

//Play a sound client side (so only client will hear it)
simulated function ClientPlaySound(sound ASound)
{
	if( LastClientSoundTime<Level.TimeSeconds )
	{
		LastClientSoundTime = Level.TimeSeconds+0.5;
		VoiceActor.PlaySound(ASound,SLOT_None,2);
		VoiceActor.PlaySound(ASound,SLOT_Misc,2);
		VoiceActor.PlaySound(ASound,SLOT_Talk,2);
	}
}

function ServerReStartGame(); // Not needed at all
function ServerReStartPlayer(); // Shouldnt be called outside dying state!
exec function ShowInventory();
function RealReStartPlayer()
{
	Super.ServerReStartPlayer();
}
event BroadcastMessage( coerce string Msg, optional bool bBeep, optional name Type ); // Not allowed to be called here!

exec function ShowPath();
exec function RememberSpot();

event PlayerInput( float DeltaTime )
{
	local float FOVScale, MouseScale;
	local byte bXAxis,bYAxis;
	
	PlayerComboTick(DeltaTime);
	LastCameraUpdate = -1;
	if( !bIsNetOwner )
		bIsNetOwner = True;
	if ( bShowMenu && (myHud != None) ) 
	{
		if ( myHud.MainMenu != None )
			myHud.MainMenu.MenuTick( DeltaTime );
		// clear inputs
		bEdgeForward = false;
		bEdgeBack = false;
		bEdgeLeft = false;
		bEdgeRight = false;
		bWasForward = false;
		bWasBack = false;
		bWasLeft = false;
		bWasRight = false;
		aStrafe = 0;
		aTurn = 0;
		aForward = 0;
		aLookUp = 0;
		aMouseX = 0;
		aMouseY = 0;
		return;
	}
	else if ( bDelayedCommand )
	{
		bDelayedCommand = false;
		ConsoleCommand(DelayedCommand);
	}
	// Check for Dodge move
	// flag transitions
	bEdgeForward = (bWasForward ^^ (aBaseY > 0));
	bEdgeBack = (bWasBack ^^ (aBaseY < 0));
	bEdgeLeft = (bWasLeft ^^ (aStrafe > 0));
	bEdgeRight = (bWasRight ^^ (aStrafe < 0));
	bWasForward = (aBaseY > 0);
	bWasBack = (aBaseY < 0);
	bWasLeft = (aStrafe > 0);
	bWasRight = (aStrafe < 0);

	// Smooth and amplify mouse movement
	FOVScale = DesiredFOV * 0.01111; // 0.01111 = 1/90
	MouseScale = MouseSensitivity * FOVScale;
	aMouseX = SmoothMouse(aMouseX*MouseScale, DeltaTime,bXAxis,0);
	aMouseY = SmoothMouse(aMouseY*MouseScale, DeltaTime,bYAxis,1);

	aMouseX = AccelerateMouse(aMouseX);
	aMouseY = AccelerateMouse(aMouseY);

	// adjust keyboard and joystick movements
	aLookUp *= FOVScale;
	aTurn   *= FOVScale;

	// Remap raw x-axis movement.
	if( bStrafe!=0 ) // strafe
		aStrafe += aBaseX * 7.5 + aMouseX;
	else // forward
		aTurn  += aBaseX * FOVScale + aMouseX;

	aBaseX = 0;

	// Remap mouse y-axis movement.
	if( (bStrafe == 0) && (bAlwaysMouseLook || (bLook!=0)) )
	{
		// Look up/down.
		if ( bInvertMouse )
			aLookUp -= aMouseY;
		else
			aLookUp += aMouseY;
	}
	else // Move forward/backward.
		aForward += aMouseY;

	if ( bSnapLevel != 0 )
	{
		bCenterView = true;
		bKeyboardLook = false;
	}
	else if (aLookUp != 0)
	{
		bCenterView = false;
		bKeyboardLook = true;
	}
	else if ( bSnapToLevel && !bAlwaysMouseLook )
	{
		bCenterView = true;
		bKeyboardLook = false;
	}

	// Remap other y-axis movement.
	if ( bFreeLook != 0 )
	{
		bKeyboardLook = true;
		aLookUp += 0.5 * aBaseY * FOVScale;
	}
	else
		aForward += aBaseY;

	aBaseY = 0;

	// Handle walking.
	HandleWalking();
}
function float SmoothMouse(float aMouse, float DeltaTime, out byte SampleCount, int Index)
{
	local int i, sum;

	if ( MouseSmoothingMode == 0 )
		return aMouse;

	if ( aMouse == 0 )
	{
		ZeroTime[Index] += DeltaTime;
		if ( ZeroTime[Index] < MouseSamplingTime )
		{
			SamplingTime[Index] += DeltaTime;
			MaybeTime[Index] += DeltaTime;
			aMouse = SmoothedMouse[Index];
		}
		else
		{
			if ( MouseSamples[Index] > 9 )
			{
				SamplingTime[Index] -= MaybeTime[Index];
				MouseSamplingTime = 0.9 * MouseSamplingTime + 0.1 * SamplingTime[Index]/MouseSamples[Index];
			}
			SamplingTime[Index] = 0;
			SmoothedMouse[Index] = 0;
			MouseSamples[Index] = 0;
		}
	}
	else
	{
		MaybeTime[Index] = 0;

		if ( SmoothedMouse[Index] != 0 )
		{
			MouseSamples[Index] += SampleCount;
			if ( DeltaTime > MouseSamplingTime * (SampleCount + 1) )
				SamplingTime[Index] += MouseSamplingTime * SampleCount;
			else
			{
				SamplingTime[Index] += DeltaTime;
				aMouse = aMouse * DeltaTime/(MouseSamplingTime * SampleCount);
			}
		}
		else
			SamplingTime[Index] = 0.5 * MouseSamplingTime;

		SmoothedMouse[Index] = aMouse/SampleCount;
		ZeroTime[Index] = 0;
	}
	SampleCount = 0;

	if ( MouseSmoothingMode > 1 )
	{
		if ( aMouse == 0 )
		{
			// stop in next tick
			for ( i=0; i<3; i++ )
			{
				sum += (i+1) * 0.1;
				aMouse += sum * OldSamples[i];
				OldSamples[i] = 0;
			}
			OldSamples[3] = 0;
		}
		else
		{
			aMouse = 0.4 * aMouse;
			OldSamples[3] = aMouse;
			for ( i=0; i<3; i++ )
			{
				aMouse += (i+1) * 0.1 * OldSamples[i];
				OldSamples[i] = OldSamples[i+1];
			}

		}
	}
	return aMouse;
}
function float AccelerateMouse(float aMouse)
{
	local float Accel;

	if ( abs(aMouse) == 0 )
		return 0;

	Accel = MouseAccelThreshold * MouseSensitivity;
	if ( abs(aMouse) < Accel )
	{
		if ( abs(aMouse) < 0.1 * Accel )
			aMouse *= 0.1;
		else
			aMouse = aMouse * abs(aMouse)/Accel;
	}
	return aMouse;
}
function rotator AdjustAim(float projSpeed, vector projStart, int aimerror, bool bLeadTarget, bool bWarnTarget)
{
	local vector Start,End,HitL,HitN,X;
	local float bestdist, bestAim;
	local Pawn ETarget;
	
	bestAim = 0.9;
	X = vector(ViewRotation);
	ETarget = PickTarget(bestAim, bestDist, X, projStart);
	if( ETarget!=None )
		ETarget.WarnTarget(Self,projSpeed,X);
	Start = Location+vect(0,0,1)*EyeHeight;
	End = Start+vector(ViewRotation)*10000;
	if( Trace(HitL,HitN,End,Start,True)==None )
		return rotator(End-projStart);
	else return rotator(HitL-projStart);
}
simulated function SwitchToMap( string NewURL )
{
	if( Level.NetMode!=NM_Client ) Return;
	ClientTravel(NewURL,TRAVEL_Relative,False);
}
function ReceiveLocalMessage( Class<LocalMessage> MsgType, optional object InvolvedObject, optional int Switch,
 optional PlayerReplicationInfo InvolvedPRI1, optional PlayerReplicationInfo InvolvedPRI2, optional string InvolvedStr )
{
	local string S;
	
	if( MsgType==None || Player==None || Player.Console==None ) Return;
	S = MsgType.Static.GetMessageString(InvolvedObject,Switch,InvolvedPRI1,InvolvedPRI2,Self,InvolvedStr);
	if( MsgType.Default.bWriteOnConsole )
	{
		if( UTHUD(myHUD)!=None )
			UTHUD(myHUD).NextMessageColor = MsgType.Static.GetMessageColor(InvolvedObject,Switch,InvolvedPRI1,InvolvedPRI2,Self,InvolvedStr);
		Player.Console.Message(InvolvedPRI1,S,'Local');
	}
	if( MsgType.Default.bMessageBeep && bMessageBeep )
		PlayBeepSound();
	if( MsgType.Default.bDisplayOnHUD && UTHUD(myHUD)!=None && Len(S)>0 )
		UTHUD(myHUD).IncomingLocalMsg(MsgType,InvolvedObject,Switch,InvolvedPRI1,InvolvedPRI2,InvolvedStr,S);
}
exec function TestMsg( string Msg )
{
	ReceiveLocalMessage(Class'CriticalMessage',,,,,Msg);
}
// Compress player movement
function CompressedMoveType CompressMove( byte XMov, byte YMov, bool bFire, bool bAltFire, bool bJump, bool bCrouch,
 bool bWalking, rotator ViewRota )
{
	local CompressedMoveType C;
	
	if( XMov<0 )
		C.PressedKeys = 1;
	else if( XMov>0 )
		C.PressedKeys = 2;
	if( YMov<0 )
		C.PressedKeys+=4;
	else if( YMov>0 )
		C.PressedKeys+=8;
	if( bFire )
		C.PressedKeys+=16;
	if( bAltFire )
		C.PressedKeys+=32;
	if( bJump )
		C.PressedKeys+=64;
	if( bCrouch )
		C.PressedKeys+=128;
	C.bPressingWalk = bWalking;
	C.ViewRot = (32767 & (ViewRota.Pitch/2)) * 32768 + (32767 & (ViewRota.Yaw/2));
	Return C;
}
// Decompress player movement
function DecompressMove( out vector Accel, out byte bFire, out byte bAltFire, out byte bJumpin, out byte bCrouchin,
 out rotator ViewRota, CompressedMoveType C )
{
	local int Pitchy;
	
	if ( (1 & C.PressedKeys)!=0 )
		Accel = vect(1,0,0);
	else if ( (2 & C.PressedKeys)!=0 )
		Accel = vect(-1,0,0);
	if ( (4 & C.PressedKeys)!=0 )
		Accel += vect(0,1,0);
	else if ( (8 & C.PressedKeys)!=0 )
		Accel += vect(0,-1,0);
	if ( (16 & C.PressedKeys)!=0 )
		bFire = 1;
	if ( (32 & C.PressedKeys)!=0 )
		bAltFire = 1;
	if ( (64 & C.PressedKeys)!=0 )
		bJumpin = 1;
	if ( (128 & C.PressedKeys)!=0 )
		bCrouchin = 1;
	ViewRota.Pitch = C.ViewRot/32768;
	ViewRota.Yaw = 2 * (C.ViewRot - 32768 * ViewRota.Pitch);
	ViewRota.Pitch *= 2;
	if( Accel==vect(0,0,0) )
		Return;
	Pitchy = ViewRota.Pitch;
	ViewRota.Pitch = 0;
	Accel = Normal(Accel >> ViewRota)*300;
	ViewRota.Pitch = Pitchy;
}
exec function ShowSpeech()
{
	if( Species==None || Species.Default.VoicePackClass==None )
		Return;
	ClientOpenSpeechMenu();
}

exec function Speech( int Type, int Index, int Callsign );

exec function SpeechSay( name Group, byte Section, int CallSig )
{
	if( SpaminCheck<Level.TimeSeconds )
	{
		if( Level.NetMode==NM_Client )
			SpaminCheck = Level.TimeSeconds+4;
		ServerSpeech(Group,Section,CallSig);
	}
}
function ServerSpeech( name Group, byte Section, int CallSig )
{
	local Pawn P;
	local PlayerReplicationInfo Rec;
	local UTGRI U;
	local int i;
	local bool bOrderAll;
	
	if( SpaminCheck>Level.TimeSeconds || PlayerReplicationInfo.bIsSpectator )
		Return;
	SpaminCheck = Level.TimeSeconds+4;
	if( Group=='Orders' && CallSig!=-2 )
	{
		U = Class'DeathMatchPlus'.Static.GetMapGRI(Self);
		bOrderAll = (CallSig==-1);
		if( U!=None && !bOrderAll )
		{
			For( i=0; i<U.NumPRIs; i++ )
			{
				if( U.ActPRIs[i]==None || U.ActPRIs[i].bIsSpectator || U.ActPRIs[i].Team!=PlayerReplicationInfo.Team
				 || U.ActPRIs[i]==PlayerReplicationInfo || U.ActPRIs[i].PlayerID!=CallSig )
					Continue;
				Rec = U.ActPRIs[i];
			}
		}
	}
	For( P=Level.PawnList; P!=None; P=P.NextPawn )
	{
		if( P.bIsPlayer && P.PlayerReplicationInfo!=None )
		{
			if ( (Group=='Taunts') || !Level.Game.bTeamGame )
				P.ClientVoiceMessage(PlayerReplicationInfo,Rec,Group,Section);
			else if ( PlayerReplicationInfo.Team == P.PlayerReplicationInfo.Team )
				P.ClientVoiceMessage(PlayerReplicationInfo,Rec,Group,Section);
			if( UTBot(P)!=None && P.PlayerReplicationInfo.Team==PlayerReplicationInfo.Team &&
			 (bOrderAll || P.PlayerReplicationInfo==Rec) )
				UTBot(P).ReceivedOrders(Section,Self);
		}
	}
}
simulated function ClientVoiceMessage(PlayerReplicationInfo Sender, PlayerReplicationInfo Recipient, name messagetype, byte messageID)
{
	local string TMsg,TLoc;
	local sound TSnd;
	
	if( UTPRI(Sender)==None || UTPRI(Sender).LoadedSpecies==None || UTPRI(Sender).LoadedSpecies.Default.VoicePackClass==None )
		Return;
	TMsg = UTPRI(Sender).LoadedSpecies.Default.VoicePackClass.Static.GetVoiceName(messagetype,messageID,TSnd);
	if( TMsg=="" && TSnd==None )
		Return;
	if( TMsg!="" )
	{
		if( Recipient!=None )
			TMsg = Class'DeathMatchPlus'.Static.ReplaceText(TMsg,"%r",Recipient.PlayerName,False);
		else TMsg = Class'DeathMatchPlus'.Static.ReplaceText(TMsg,"%r","",False);
		if( Sender!=None && Sender.PlayerZone!=None && Sender.PlayerZone.ZoneName!="" )
			TLoc = "("$Sender.PlayerZone.ZoneName$")";
		TMsg = Class'DeathMatchPlus'.Static.ReplaceText(TMsg,"%z",TLoc,False);
		if( TMsg!="" && myHUD != None )
			myHUD.Message( Sender, TMsg, 'Say'); // Don't beep message nor log on console!
	}
	if( TSnd!=None && LastSoundTimerr<Level.TimeSeconds )
	{
		LastSoundTimerr = Level.TimeSeconds+0.8;
		VoiceActor.PlaySound(TSnd,SLOT_Talk,1.8,,50000);
	}
}

exec function SetChar( string S )
{
	local int i;
	local string ch;
	
	if( S=="" )
	{
		ClientMessage("SetChar useage: SetChar <species class> <skin name>");
		Return;
	}
	i = InStr(S," ");
	if( i==-1 )
	{
		ClientMessage("Missing character skin");
		Return;
	}
	ch = Left(S,i);
	S = Mid(S,i+1);
	if( class<PlayerSpecies>(DynamicLoadObject(ch,Class'Class'))==None )
	{
		Log("Missing character class '"$ch$"'");
		Return;
	}
	ServerApplyCharacter(ch,S);
}

function ServerApplyCharacter( string ch, string skinname )
{
	if( LastCharacterChangeTime>Level.TimeSeconds )
		Return;
	LastCharacterChangeTime = Level.TimeSeconds+4;
	if( Health>0 )
		KilledBy(None);
	SetMeshSkin(skinname,ch);
}

exec function Ping()
{
	ClientTellPing();
}
function ClientTellPing()
{
	ClientMessage("Current ping is"@(ExactPing*100.f)@"MS");
}

simulated function ClientHearSound( actor Actor, int Id, sound S, vector SoundLocation, vector Parameters )
{
	if( ModifierList!=None && !ModifierList.HearSound(Actor,Id,S,SoundLocation,Parameters) )
		Return;
	Super.ClientHearSound(Actor,Id,S,SoundLocation,Parameters);
}

/////////////////////////////////////////////////////////////////////////
// Administrative commands:

function bool CanAccessCmd( name CmdName, optional string Parms )
{
	if( Level.Netmode==NM_Standalone )
		Return True;
	if( DeathMatchPlus(Level.Game)==None || DeathMatchPlus(Level.Game).AccessHandler==None )
		Return bAdmin;
	Return DeathMatchPlus(Level.Game).AccessHandler.CanExecuteCmd(Self,CmdName,Parms);
}

exec function Walk()
{	
	if ( !CanAccessCmd('Walk') )
		return;
	StartWalk();
}
exec function Ghost()
{
	if ( !CanAccessCmd('Ghost') )
		return;
	
	UnderWaterTime = -1.0;	
	ClientMessage("You feel ethereal");
	SetCollision(false, false, false);
	bCollideWorld = false;
	GotoState('CheatFlying');
}
function ServerAddBots(int N)
{
	local int i;

	if ( !Level.Game.bDeathMatch || !CanAccessCmd('AddBots',string(N)) )
		return;
	DeathMatchGame(Level.Game).InitialBots = Level.Game.NumPlayers+DeathMatchGame(Level.Game).NumBots+N;
	for ( i=0; i<N; i++ )
		Level.Game.AddBot();
}
exec function KillBots( optional int N )
{
	local Pawn P;
	
	if ( !Level.Game.bDeathMatch || !CanAccessCmd('KillBots',string(N)) )
		return;
	if( N==0 )
	{
		N = DeathMatchGame(Level.Game).NumBots;
		DeathMatchGame(Level.Game).InitialBots = 0;
	}
	DeathMatchGame(Level.Game).InitialBots = Level.Game.NumPlayers+DeathMatchGame(Level.Game).NumBots-N;
	For( P=Level.PawnList; P!=None; P=P.NextPawn )
	{
		if( P.IsA('Bots') )
		{
			P.Destroy();
			N--;
			if( N<=0 ) Return;
		}
	}
}
function StartWalk()
{
	if( Region.Zone!=None && Region.Zone.bWaterZone )
		GoToState('PlayerSwimming');
	else GoToState('PlayerWalking');
	Super.StartWalk();
}
function ServerSetSloMo(float T)
{
	if ( CanAccessCmd('SloMo',string(T)) )
		Level.Game.SetGameSpeed(T);
}
exec function AllAmmo()
{
	ServerAllAmmo();
}
function ServerAllAmmo()
{
	local Inventory Inv;

	if ( !CanAccessCmd('AllAmmo') )
		return;

	for( Inv=Inventory; Inv!=None; Inv=Inv.Inventory ) 
		if (Ammo(Inv)!=None) 
		{
			Ammo(Inv).AmmoAmount  = 999;
			Ammo(Inv).MaxAmmo  = 999;				
		}
}
exec function God()
{
	if ( !CanAccessCmd('God') )
		return;

	if ( ReducedDamageType == 'All' )
	{
		ReducedDamageType = '';
		ClientMessage("God mode off");
		return;
	}

	ReducedDamageType = 'All'; 
	ClientMessage("God Mode on");
}
exec function Invisible(bool B)
{
	ServerInvisible(B);
}
function ServerInvisible(bool B)
{
	if ( !CanAccessCmd('Invisible',string(B)) )
		return;

	if (B)
	{
		bHidden = true;
		Visibility = 0;
	}
	else
	{
		bHidden = false;
		Visibility = Default.Visibility;
	}	
}
exec function SetJumpZ( float F )
{
	ServerSetJumpZ(F);
}
function ServerSetJumpZ( float F )
{
	if ( !CanAccessCmd('SetJumpZ',string(F)) )
		return;
	JumpZ = F;
}
exec function SetSpeed( float F )
{
	ServerSetSpeed(F);
}
function ServerSetSpeed( float F )
{
	if ( !CanAccessCmd('SetSpeed',string(F)) )
		return;
	GroundSpeed = Default.GroundSpeed * f;
	WaterSpeed = Default.WaterSpeed * f;
}
exec function KillAll(class<actor> aClass)
{
	local Actor A;

	if ( aClass==None || !CanAccessCmd('KillAll',string(aClass)) )
		return;
	ForEach AllActors(aClass, A)
		if ( PlayerPawn(A)==None && A!=Level.Game )
			A.Destroy();
}

exec function KillPawns()
{
	local Pawn P;
	
	if ( !CanAccessCmd('KillPawns') )
		return;
	ForEach AllActors(class 'Pawn', P)
		if (PlayerPawn(P) == None)
			P.Destroy();
}

exec function Summon( string ClassName )
{
	local class<actor> NewClass;
	local float Dist;
	
	if ( !CanAccessCmd('Summon',ClassName) )
		return;
	if( InStr(ClassName,".")==-1 )
		ClassName = "UnrealI."$ClassName;
	NewClass = class<actor>( DynamicLoadObject( ClassName, class'Class' ) );
	if( NewClass==None )
		ClientMessage("Cant find class '"$ClassName$"'");
	else if( NewClass.Default.bNoDelete )
		ClientMessage("Can't spawn that desired actor becouse it has bNoDelete");
	else if( NewClass.Default.bStatic )
		ClientMessage("Can't spawn that desired actor becouse it has bStatic");
	else
	{
		Dist = NewClass.Default.CollisionRadius+25;
		if( Dist<72 )
			Dist = 72;
		if( Spawn( NewClass,,,Location + Dist*Vector(ViewRotation) + vect(0,0,1)*15,ViewRotation)==None )
			ClientMessage("Failed to spawn that actor, possible becouse it has no space or actor is abstract.");
	}
}
exec function Profile();
exec function Admin( string CommandLine )
{
	local string Result;
	local int l;
	
	if ( !CanAccessCmd('Admin',CommandLine) )
		Return;
	Result = ConsoleCommand(CommandLine);
	if( Result=="" )
		Return;
	l = Len(Result);
	While( l>250 )
	{
		ClientMessage(Left(Result,250));
		Result = Mid(Result,250);
		l-=250;
	}
	ClientMessage(Result);
}
exec function PlayerList();
exec function ShowSpecialMenu( string ClassName );
exec function CauseEvent( name N )
{
	SendEvent(N);
}
exec function SendEvent( name N )
{
	local actor A;
	if( CanAccessCmd('CauseEvent',string(N)) && (N != '') )
		foreach AllActors( class 'Actor', A, N )
			A.Trigger( Self, Self );
}
exec function CallForHelp();
exec function RestartLevel();
exec function LocalTravel( string URL );

exec function Kick( string S ) 
{
	local Pawn aPawn;
	local int ID;
	
	ID = int(S);
	if ( !CanAccessCmd('Kick',string(ID)) )
		return;
	for( aPawn=Level.PawnList; aPawn!=None; aPawn=aPawn.NextPawn )
		if
		(	aPawn.bIsPlayer
			&&	aPawn.PlayerReplicationInfo.PlayerID==ID 
			&&	(PlayerPawn(aPawn)==None || NetConnection(PlayerPawn(aPawn).Player)!=None ) )
		{
			aPawn.Destroy();
			return;
		}
}
exec function GetWeapon(class<Weapon> NewWeaponClass )
{
	ServerGetWeapon(NewWeaponClass);
}
exec function ServerGetWeapon(class<Weapon> NewWeaponClass )
{
	local Inventory Inv;

	if ( (Inventory == None) || (NewWeaponClass == None)
		|| ((Weapon != None) && (Weapon.Class == NewWeaponClass)) )
		return;

	for ( Inv=Inventory; Inv!=None; Inv=Inv.Inventory )
		if ( Inv.Class == NewWeaponClass )
		{
			PendingWeapon = Weapon(Inv);
			Weapon.PutDown();
			return;
		}
}
exec function PlayersOnly()
{
	ServerPlayersOnly();
}
function ServerPlayersOnly()
{
	if ( !CanAccessCmd('PlayersOnly') )
		return;

	Level.bPlayersOnly = !Level.bPlayersOnly;
}
exec function SwitchLevel( string URL )
{
	if ( CanAccessCmd('SwitchLevel',URL) )
		Level.ServerTravel( URL, false );
}
exec function SwitchCoopLevel( string URL )
{
	if ( CanAccessCmd('SwitchCoopLevel',URL) )
		Level.ServerTravel( URL, true );
}

exec function Login( string Password )
{
	if( DeathMatchPlus(Level.Game)!=None && DeathMatchPlus(Level.Game).AccessHandler!=None )
		DeathMatchPlus(Level.Game).AccessHandler.ClientAdminLogin(Self,Password);
}
exec function Logout()
{
	if( DeathMatchPlus(Level.Game)!=None && DeathMatchPlus(Level.Game).AccessHandler!=None )
		DeathMatchPlus(Level.Game).AccessHandler.ClientAdminLogout(Self);
}
event ClientMessage( coerce string S, optional Name Type, optional bool bBeep )
{
	if( Len(S)==0 )
		Return;
	Super.ClientMessage(S,Type,bBeep);
}
event Possess()
{
	Super.Possess();
	if( Viewport(Player)!=None && VoiceActor==None )
		VoiceActor = Spawn(Class'VoicActor',Self);
}
exec function Taunt( name Sequence )
{
	if( VSize(Acceleration)<5 && Physics==PHYS_Walking && !bIsCrouchingNow )
	{
		Switch( Sequence )
		{
		case 'Taunt1':
			AnimAction = 1;
			Break;
		case 'Victory1':
			AnimAction = 2;
			Break;
		case 'Wave':
			AnimAction = 3;
			Break;
		default:
			Return;
		}
		AnimResetTime = Level.TimeSeconds+0.2;
		if( Level.NetMode==NM_Client )
			ServerTaunt(Sequence);
	}
}
function ServerTaunt( name Sequence )
{
	if( VSize(Acceleration)<5 && Physics==PHYS_Walking && !bIsCrouchingNow )
	{
		Switch( Sequence )
		{
		case 'Taunt1':
			AnimAction = 1;
			Break;
		case 'Victory1':
			AnimAction = 2;
			Break;
		case 'Wave':
			AnimAction = 3;
			Break;
		default:
			Return;
		}
		AnimResetTime = Level.TimeSeconds+0.2;
	}
}
exec function ActivateItem()
{
	if( bShowMenu || Len(Level.Pauser)>0 )
		return;
	if( !DeathMatchPlus(Level.Game).OverrideActivateKey(Self) && SelectedItem!=None)
		SelectedItem.Activate();
}
simulated function ClientOpenUWinMenu( class<UWindowFramedWindow> MenuClass, float XPos, float YPos, float XSize, float YSize )
{
	local WindowConsole W;

	if( Player==None || WindowConsole(Player.Console)==None || MenuClass==None )
		return;
	W = WindowConsole(Player.Console);
	W.Root.CreateWindow(MenuClass,W.Root.RealWidth*XPos,W.Root.RealHeight*YPos,W.Root.RealWidth*XSize,W.Root.RealWidth*YSize,,True);
	W.bQuickKeyEnable = True; 
	W.LaunchUWindow();
}
function ServerCustomCMD( optional name Nm, optional int Swit, optional Object Obj )
{
	DeathMatchPlus(Level.Game).ClientDidCMD(Self,Nm,Swit,Obj);
}

defaultproperties
{
				PlayingTrack=255
				RenderingClass=Class'BIH.PawnCRender'
				SavedMoveClass=Class'BIH.UTSavedMove'
				MaxJumps=1
				JumpZModifer=-50.000000
				MouseSamplingTime=0.008333
				VoicePackMenuClass=Class'BIH.VoicePackMenu'
				bPlayMP3Songs=True
				MP3SongVolume=1.000000
				MinAdrenalineCost=100
				ComboList(0)=Class'BIH.BoosterCombo'
				ComboList(1)=Class'BIH.SpeedCombo'
				ComboList(2)=Class'BIH.InvisibilityCombo'
				ComboList(3)=Class'BIH.BerserkCombo'
				CarcassType=Class'UnrealI.MaleOneCarcass'
				bSinglePlayer=True
				GroundSpeed=400.000000
				AirSpeed=400.000000
				AccelRate=2048.000000
				BaseEyeHeight=34.000000
				UnderWaterTime=20.000000
				MenuName="UT Player"
				PlayerReplicationInfoClass=Class'BIH.UTPRI'
				DrawType=DT_Mesh
				CollisionRadius=17.000000
				CollisionHeight=39.000000
				Buoyancy=99.000000
}
